' fNum32Ang.bas for FilmStar DESIGN 32 bit
' Copyright 2001 FTG Software Associates
' Modification of fNum32.bas 11/07/01
'
' ****************************************************
'    Warning: Save modified program with new name!!!
' ****************************************************
'
' Use with FILM Archive FNUM.
' This program simulates the performance of a filter in a convergent
' light beam with uniform illumination and given f-number. It does this
' by repeating the calculation over an angle range and averaging.
' The initial incident angle can be non zero.
'
' To run this program as a macro, insert the following under Evaluate...Macro...Edit
' (adjusting if your winfilm directory is different)
'
'   Description   F Number
'   Command       basrun c:\winfilm\basic\f number.bas;
'
DefInt I-n
DefSng A-H, O-Z
Public Fnum, nRes, Angle0, kSource
Option Explicit
Const RADDEG = 57.29578
Const PI2 = 6.283185

'  FilmStar DESIGN Key Words
'
'   Angle.......Angle of incidence in degrees
'   Angle = 5.23
'
'   Pol.........Polarization P, S, R
'   Pol = "R"
'
'   Macro.......Call DESIGN macro function (see DDE help)
'   kErr = Macro("Calcplot;")
'
Sub GetAngle  ' dialog box for entering f number and resolution
    Dim ang
	On Error GoTo Quit   ' cancel button causes error
	' Highlight the lines Begin Dialog...End Dialog and activate the
    ' built-in dialog editor. The dialog box will appear in the editor.
    Begin Dialog UserDialog 430,207,"F-number (Cone Angle) Calculator",.DialogFunc ' %GRID:5,3,1,1
        Text 30,20,100,12,"&F Number",.lbl1,1
        TextBox 140,18,80,18,.txtFnum
        Text 30,50,100,12,"&Cone half-angle",.lbl2,1
        TextBox 140,48,80,18,.txtCone
        Text 230,51,40,12,"deg",.Text2
        Text 30,80,100,12,"&Substrate tilt",.lbl3,1
        TextBox 140,78,80,18,.txtTilt
        Text 230,81,35,12,"deg"
        Text 30,110,100,12,"&Resolution",.lbl4,1
        TextBox 140,108,50,18,.txtResol
        OKButton 305,18,90,24
        CancelButton 305,54,90,24
        GroupBox 40,141,255,51,"Source Type",.fraSource
        OptionGroup .optSource
            OptionButton 60,165,85,12,"&1 Uniform",.optUniform
            OptionButton 170,165,110,15,"&2 Lambertian",.optLambert
    End Dialog
    Dim dlg As UserDialog
    If Fnum = 0 Then Fnum = FnumGet(5)
    If nRes = 0 Then nRes = 10

C1:
	Dialog dlg
    Fnum=Val(dlg.txtFnum)   ' values OK??
	If Fnum < .5 Then
		Beep
        MsgBox "F-number must be > 0.5", 48, "FNUM Macro"
		GoTo C1
	End If
    nRes=Int(Val(dlg.txtResol))
	If nRes < 1 Then
		Beep
        MsgBox "Resolution must be > 0", 48, "FNUM Macro"
		GoTo C1
    End If
    kSource = dlg.optSource ' 0 uniform, 1 Lambertian
    ang = Val(dlg.txtTilt)
    If ang<0 Or ang>=90 Then
        Beep
        MsgBox "Angle must be 0 to 90", 48, "FNUM Macro"
        GoTo c1
    Else
        Angle = ang
    End If
	Exit Sub
Quit:
	End
End Sub

Sub Main
    On Error GoTo ErrTrap
	Do
		GetAngle
		Evaluate
	Loop
    End
ErrTrap:
    MsgBox Err.Description, vbOkOnly + vbCritical, "FNUM Macro"
	End
End Sub

Sub Evaluate   ' Does the calculation
	Dim xdata(), ydata(), ytmp(), yd()
    Dim j, k, n, nData, a, ang, ang0, tilt, delR, rMax
    Dim nInt, sa1, sa2, sina, theta, t0, wJ, wTot
    AxesDraw
    CalcPlotStart
    LineColor "B"
    a=1/(2*Fnum)
	rMax = a/Sqr(1-a^2)		' normalized
	delR = rMax/nRes
    Pol = "R"   ' random polarization
    ang0 = Angle
    tilt = ang0 / RADDEG  ' get initial tilt angle
    For n = 1 To nRes     ' integrate over radii
        StatusLabel "Iteration " & CStr(n)
        ang = Atn(Sqr(((delR * n)^2 + (delR*(n - 1))^2)/2))       ' mean illumination angle (radians)
        Select Case kSource
        Case 0  ' uniform
            wJ = ((delR*n)^2 - (delR*(n-1))^2)/rMax^2             ' uniform illumination
        Case 1  ' Lambertian
            wJ = Cos(ang) * ((delR*n)^2 - (delR*(n-1))^2)/rMax^2  ' Lambertian illumination (cosine)
        ' add other cases as required
        End Select
        t0 = PI2 / nRes
        sa1 = Sin(ang) * Sin(tilt)
        sa2 = Cos(ang) * Cos(tilt)
        If tilt > 0 Then nInt = nRes Else nInt = 1
        For j = 1 To nInt   ' integrate over circle
            If nInt = 1 Then
                theta = 0
            Else
                theta = (j -.5) * t0
            End If
            sina = sa1 * Sin(theta) + sa2
            Angle = 90 - RADDEG * Atn(sina / Sqr(1 - sina^2)) ' angle hitting substrate
            wTot = wTot + wJ    ' total energy
            Calculate
            ydata = Spectrum_Y  ' Get spectrum
            nData = UBound(ydata, 1)
            If n = 1 And j = 1 Then ReDim yd(1 To nData, 2)
            For k = 1 To nData
                yd(k, 1) = yd(k, 1) + wJ * ydata(k, 1)
                yd(k, 2) = yd(k, 2) + wJ * ydata(k, 2)
            Next k
        Replot
        Next j
    Next n
	For j = 1 To nData
		ydata(j, 1) = yd(j, 1) / wTot   ' Reflectance
		ydata(j, 2) = yd(j, 2) / wTot   ' Transmittance
	Next j
	Spectrum_Y = ydata  ' Let spectrum
	LineColor "R"
	Replot
    CalcPlotDone
    Angle = ang0    ' restore original value
End Sub

Function DialogFunc%(DlgItem$, Action%, SuppValue%)
    Dim s, s1$, s2$
    Select Case Action%
    Case 1 ' initialization
        DlgText "txtTilt", CStr(Angle)  ' current angle of incidence
        DlgText "txtFnum", CStr(Fnum)
        DlgText "txtCone", CStr(ConeGet(Fnum))
        DlgText "txtResol", CStr(nRes)
        DlgValue "optSource", kSource
    Case 3 ' text box change
        Select Case DlgItem
        Case "txtFnum"
            DlgText "txtCone", CStr(ConeGet(Val(DlgText$("txtFnum"))))
        Case "txtCone"
            DlgText "txtFnum", CStr(FnumGet(Val(DlgText$("txtCone"))))
        Case Else
        End Select
    End Select
End Function

Function FnumGet(ByVal a)   ' a is cone angle in deg
    On Error Resume Next
    FnumGet = 1/(2 * Sin(a / RADDEG))
End Function

Function ConeGet(ByVal f)  ' f is fnumber
    Dim sa
    On Error Resume Next
    sa = 0.5 / f
    ConeGet = RADDEG * Atn(sa/Sqr(1 - sa^2))
End Function
