' Spector2.bas for FilmStar DESIGN
' Copyright 2020 FTG Software Associates
' Creates coating file for Veeco Spector OMS
' Version 1 Layers only
' Version 2 adds Cauchy dispersion

Option Base 1
DefInt i-n
DefSng A-H, O-Z

' ***************** Change as required ****************************
Const fPath$ = PgmPath & "\Spector"    ' usually c:\Winfilm\Spector
Const Title$ = "Spector OMS Coating File"
Const Version = 2
' *****************************************************************

Option Explicit

Sub Main
	Dim i, n, s$, b As Boolean
	Dim q(), qt$(), qd$()
	Dim file$, nSymb$(), nTyp$(), nFile$(), nDesc$(), Des0$, nMatl
	On Error GoTo MainQuit
	Des0$ = Design  ' Initial routines to check settings
	If InStr(Des0$, "(") > 0 Then
		MsgBox "Groups Mode designs not permitted; convert to Layers", vbExclamation, Title$
		End
	End If
	If DesignQwotUnit = .25 Then
		MsgBox "Change Optical Thickness Convention" & vbLf & "to QWOT = 1.0 in Setup...Parameters", vbExclamation, Title$
		End
	End If
	GetIndexList nSymb$(), nTyp$(), nFile$(), nDesc()
	nMatl = UBound(nSymb$)
	For i = 2 To UBound(nSymb$)
		If i=2 Or InStr(Des0$, nSymb$(i)) > 0 Then
			If i > 2 And nTyp$(i) <> "Opt" Then
				MsgBox "Layers must be in optical thickness" & vbLf & "Click Design...Physical to Optical", vbExclamation, Title$
				End
			End If
			If nFile$(i) <> "$CAUCHYM" Then
				MsgBox "Substrate and materials require CAUCHYM dispersion." & vbLf & "Change in Setup...Film Indices", vbExclamation, Title$
				End
			End If
		End If
	Next i

GetFileName:
	file$ = GetFilePath$(,"fsd", fPath$, "Open " & Title$, 2)  ' cancel quits program
	If FileExists(file$) Then
		Select Case MsgBox("Overwrite  " & file$, vbQuestion + vbYesNoCancel, "File Exists")
		Case vbYes
			Kill file$
		Case vbNo
			GoTo GetFileName
		Case vbCancel
			GoTo MainQuit
		End Select
	End If
	Open file$ For Output As #1
	On Error Resume Next
	Print #1, "ITISPECTORDESIGN"
	Print #1, "VERSION:"&tFmt$(Version, 1)
	Print #1, "DESIGNWAVELENGTH:" & tFmt$(DesignWave, 2) ' Nanometers
	If Version >=2 Then
		For i = 2 To nMatl
			If i = 2 Then
				s$ = "SUBSTRATE": b = True
			ElseIf InStr(Des0$, nSymb$(i)) > 0 Then	' Ignore unused materials
				s$ = "MATERIAL:" & nSymb$(i): b = True
			Else
				b = False
			End If
			If b Then Print #1, s$ & ":CAUCHY:" & tStr$(IndexCoeff(i, 1)) & ":" & tStr$(IndexCoeff(i, 2)) & ":" & tStr$(IndexCoeff(i,3))
		Next i
	End If
	GetLayers n, q(), qt$(), qd$()
	Print #1, "LAYERS:" & CStr(n)
	For i = 1 To n
		Print #1, "LAYER:" & CStr(i)& ":" & qt$(i) & ":" & tFmt$(q(i), 5)
	Next i
	Close
MainQuit:
	Busy False
	End
End Sub

Function FileExists(ByVal f$) As Boolean
    On Error GoTo NoFile
    FileExists = FileLen(f$) > 0
    Exit Function
NoFile:
    FileExists = False
End Function

Function tStr$(ByVal v)
	tStr$ = Trim$(Str$(v))
End Function

Function tFmt$(ByVal v, ByVal n)	' replaces "," with "," in case of European notation
	tFmt$=Replace(Format$(v, "0." & String(n,"0")), ",", ".")
End Function
