' SpektrumNK.bas for FilmStar INDEX exports LZH .dsp files for CEC coaters
' Copyright 2018 FTG Software Associates
Option Explicit
Option Base 1
DefInt i-N
DefSng A-H, O-Z
Public CecFile$, CecPath$, fType$, Coeff()
Public Const mTitle$ = "Spektrum (CEC) Index File"

Sub Main
    Dim k, sFile$
    If InStr(PgmCaption$, "INDEX") = 0 Then
        MsgBox "This program must be run from FilmStar INDEX", vbCritical, mTitle$
        End
    End If
    If IndexName$ = "" Then
        MsgBox "Open an INDEX dispersion file (*.itw) and try again", vbExclamation, mTitle$
        End
    End If
    If MsgBox("Export n,k File  " & IndexName$ & ".itw to .dsp?", vbOkCancel + vbQuestion, mTitle$) = vbCancel Then End
    ' ********* Change as required **********
    '   PgmPath$ is normally C:\Winfilm
        CecPath$ = PgmPath$ & "\Export"
    ' ***************************************
    If Not PathExists(CecPath$) Then MkDir CecPath$
    CecFile$ = CecPath$ & "\" & IndexName$ & ".dsp"

    If FileExists(CecFile$) Then
        Select Case MsgBox(CecFile$ & " already exists!" & vbLf & vbLf &  "Replace it? <No> saves with new name." , _
          vbYesNoCancel + vbExclamation + vbDefaultButton2+ vbQuestion, mTitle$)
        Case vbYes
            Kill CecFile$
        Case vbNo
            Do
                k = k + 1
                sFile$ = Left$(CecFile$, Len(CecFile$)-4) & CStr(k) & ".dsp"
            Loop Until Not FileExists(sFile$)
            CecFile$ = sFile$
        Case vbCancel
            CloseDown
            End
        End Select
    End If
    Open CecFile$ For Output As #1
    AddMaterial
    CloseDown
    MsgBox CecFile$ & " saved", vbInformation, mTitle$
End Sub

Sub CloseDown
    On Error Resume Next
    Close #1
    DoEvents
End Sub

Sub AddMaterial
    fType$ = FunctionType
    If fType$<> "" Then Coeff = FunctionCoeff
    Print #1, "##TITLE=" & IndexName$
    Print #1, "##INFO="
    Select Case fType$
    Case ""
        AddNKtable1
    Case "CE-CY6", "CE-CY6K"
        AddCauchy Coeff(1), Coeff(2), Coeff(3), Coeff(4), Coeff(5), Coeff(6)
        NoTable
    Case "CE-SM6", "CE-SM6K
        AddSellmeier Coeff(1), Coeff(2), Coeff(3), Coeff(4), Coeff(5), Coeff(6)
        NoTable
    Case Else ' User-Defined with Spektrum K-function
        If FunctionCode$<>"N1,A*EXP(B/WM+C*WM)" Then    ' Required k-Formula code
            MsgBox "User-Defined Index Function " & fType$ & " is not Spektrum compatible", vbCritical, mTitle$
            CloseDown
            End
        End If
        NoSellmeier
        NoCauchy
        NoOscillator
        AddKformula Coeff(1), Coeff(2), Coeff(3)
        AddNKtable2 UserIndxTbl(1, fType$)
    Case Else
        MsgBox "Built-in Index function " & fType$ & " is not Spektrum compatible", vbCritical, mTitle$
        CloseDown
        End
    End Select
    DoEvents
    Exit Sub
End Sub

Sub NoTable()
    If InStr(fType$, "K") = 0 Then
        NoKformula
    Else
        AddKformula Coeff(7), Coeff(8), Coeff(9)
    End If
    Print #1, "##RATE=0"
    If InStr(fType$, "K") = 0 Then
        Print #1, "##K_WERTE=FALSE"
    Else
        Print #1, "##K_WERTE=K-FORMULA"
    End If
    Print #1, "##NPOINTS=0"
    Print #1, "##DATEN(L_n_k)=" & vbCrLf   ' add blank line
End Sub

Sub AddNKtable1()
    Dim i, s$, nVals, vnk!(), isAbs As Boolean
    NoSellmeier
    NoCauchy
    NoOscillator
    NoKformula
    Print #1, "##RATE=0"
    vnk = nkTable
    DoEvents
    For i=1 To UBound(vnk, 1)
        If vnk(i, 3) > 0 Then
            isAbs = True
            Exit For
        End If
    Next i
    If isAbs Then
        Print #1,"##K_WERTE=TABLE"
    Else
        Print #1,"##K_WERTE=FALSE"
    End If
    Print #1, "##NPOINTS=" & CStr(nIndex)
    Print #1, "##DATEN(L_n_k)="
    Print #1, IndexList(vbTab, 1) & vbCrLf
End Sub

Sub AddNKtable2(ByVal f$)
    Dim i, s$, nVals, vnk!(), isAbs As Boolean
    If f$ = "" Then Exit Sub
    Print #1, "##RATE=0"
    IndexOpen f$
    vnk = nkTable
    DoEvents
    Print #1, "##K_WERTE=K-FORMULA"
    Print #1, "##NPOINTS=" & CStr(nIndex)
    Print #1, "##DATEN(L_n_k)="
    Print #1, IndexList(vbTab, 1) & vbCrLf
End Sub

Sub AddKformula(ByVal B1, ByVal B2, ByVal B3)
    Print #1, "##K-KOEFF="
    Print #1, CStr(B1)
    Print #1, CStr(B2)
    Print #1, CStr(B3)
End Sub

Sub AddCauchy(ByVal A0, ByVal A1, ByVal A2, ByVal A3, ByVal A4, ByVal A5)
    Dim i
    NoSellmeier
    Print #1, "##CAUCHY-KOEFF=TRUE"
    Print #1, CStr(A0)
    Print #1, CStr(A1)
    Print #1, CStr(A2)
    Print #1, CStr(A3)
    Print #1, CStr(A4)
    Print #1, CStr(A5)
    NoOscillator
End Sub

Sub AddSellmeier(ByVal B1, ByVal B2, ByVal B3, ByVal C1, ByVal C2, ByVal C3)
    Print #1, "##SM-KOEFF=TRUE"
    Print #1, CStr(B1) & vbTab & CStr(C1)
    Print #1, CStr(B2) & vbTab & CStr(C2)
    Print #1, CStr(B3) & vbTab & CStr(C3)
    NoCauchy
    NoOscillator
End Sub

Sub NoSellmeier
    Dim i
    Print #1, "##SM-KOEFF=FALSE"
    For i=1 To 3
        Print #1, "0" & vbTab & "0"
    Next i
End Sub

Sub NoCauchy
    Dim i
    Print #1, "##CAUCHY-KOEFF=FALSE"
    For i=1 To 6
        Print #1, "0"
    Next i
End Sub

Sub NoOscillator
    Dim i
    Print #1, "##OM-KOEFF=FALSE"
    For i=1 To 3
        Print #1, "0" & vbTab & "0" & vbTab & "0"
    Next i
End Sub

Sub NoKformula
    Dim i
    Print #1, "##K-KOEFF="
    For i=1 To 3
        Print #1, "0"
    Next i
End Sub

Function SMeval(ByVal W, ByVal B1, ByVal C1, ByVal B2, ByVal C2, ByVal B3, ByVal C3)
    ' Test routine, not utilized in dsp file
    ' Sellmeier...same as SELLG glass formula
    Dim W2
    W = .001*W
    W2 = W*W
    SMeval = Sqr(1 + B1*W2/(W2-C1) + B2*W2/(W2-C2) + B3*W2/(W2-C3))
End Function

Function CYeval(ByVal W, ByVal A0, ByVal A1, ByVal A2, ByVal A3, ByVal A4, ByVal A5)
    ' Test routine, not utilized in dsp file
    ' Cauchy...usually only 2-3 terms are required
    W = .001 * W
    CYeval = A0 + A1/W^2 + A2/W^4 + A3/W^6 + A4/W^8 + A5/W^10
End Function

Function PathExists(ByVal d$) As Boolean
    Dim sdum$
    On Error Resume Next
    If Trim$(d$) <> "" Then
        If Right(d$, 1) <> "\" Then d$ = d$ & "\"
        sdum$ = Dir$(d$ & "*.*", vbDirectory)
        PathExists = (sdum$ <> "")
    Else
        PathExists = False
    End If
End Function

Function FileExists(ByVal f$) As Boolean
    On Error GoTo NoFileFound
    If Trim$(f$)="" Then
        FileExists = False
    Else
        FileExists = (FileLen(f$) > 0)
    End If
    Exit Function
NoFileFound:
    FileExists = False
    Exit Function
End Function
